import requests
from datetime import datetime
import os, sys
import subprocess


class _InnerClass:
    def __init__(self):
        self.access_token = os.getenv("SLACK_ACCESS_TOKEN")
        self.channel_id = os.getenv("SLACK_CHANNEL_ID")
        self.thread_ts = ""


    def set_access_token(self, access_token):
        self.access_token = access_token


    def set_channel_id(self, channel_id):
        self.channel_id = channel_id


    def reset_thread(self):
        self.thread_ts = ""


    def _post(self, text, info = {}, url="https://slack.com/api/chat.postMessage"):
        time = str(datetime.now())
        info = "\n".join(f'{k}:\t{v}' for k, v in info.items())
        headers = {
            "content-type": "application/json",
            'Authorization': f'Bearer {self.access_token}'
        }

        try:
            return requests.post(url,
                headers = headers,
                json = {
                    "channel": self.channel_id,
                    "text": f"{text}\ntime:\t{time}\n{info}",
                    "thread_ts": self.thread_ts,
                }
            )
        except requests.exceptions.RequestException as e:
            print(f"post error!\n{e}", file=sys.stderr)
            return None


    def start_thread(self, text="start :tada:", params={}, **kwargs):
        response = self._post(text = text, info = {**params, **kwargs})
        self.thread_ts = response.json()["ts"]
        return response


    def ping(self, text="ping!", params={}, **kwargs):
        return self._post(text = text, info = {**params, **kwargs})


    def wrap(self, function, start_message="start! :sparkles:", end_message="end! :confetti_ball:"):
        def inner(*args, **kargs):
            self._post(start_message)
            try:
                return function(*args, **kargs)
            except Exception as e:
                self._post(e)
                raise e
            self._post(end_messageend_message)
        return inner


DEADBEATS = _InnerClass()


def main():
    # print("DEADBEATS")
    # print("☠️ see https://youtu.be/6ydgEipkUEU")
    start = datetime.now()
    DEADBEATS.start_thread()
    try:
        subprocess.run(sys.argv[1:], shell=True)
    except Exception as e:
        DEADBEATS.ping(e)
        raise e
    end = datetime.now()
    DEADBEATS.ping(text=f"end :sparkles:\nduration:\t{end - start}")


if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['deadbeats']
install_requires = \
['datetime>=4.3,<5.0', 'requests>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['deadbeats = deadbeats:main']}

setup_kwargs = {
    'name': 'deadbeats',
    'version': '0.3.1',
    'description': '',
    'long_description': '# DEADBEATS\n\nAn easy to use Slack messaging library for research.\n\n## Usage\n\n```python\nfrom deadbeats import DEADBEATS\n# set environment variables as below\n# SLACK_ACCESS_TOKEN=xxxx-xxxxxxxxxxxxx-xxxxxxxxxxxxxx-xxxxxxxxxxxxxxxxxxxxxxxx (Get your own Slack API access token)\n# SLACK_CHANNEL_ID=deadbeats (set slack channel id whatever you like!)\n\n\n# or you can set configurations manually.\nDEADBEATS.set_access_token("SLACK_ACCESS_TOKEN")\nDEADBEATS.set_channel_id("SLACK_CHANNEL_ID")\n\n\n# `DEADBEATS.wrap` sends a message at the beginning and end of the function.\n# `DEADBEATS.wrap` catch every errors and raise it after sending a error message.\n@DEADBEATS.wrap\ndef main():\n    # A simple "heartbeating" message.\n    DEADBEATS.ping()\n\n\n    # Start threading!\n    # All subsequent messages will be sent to the thread.\n    DEADBEATS.start_thread()\n\n\n    # You can add extra information like below.\n    params = {"loss": 0.5, "val_loss": 1.6, "acc": 100.0}\n    DEADBEATS.ping(text="message whatever you like", params=params, additional="info", huga="huga")\n\n\n    # If you want to stop threading, you can use this method.\n    # This method reset "thread_ts" of a instance variable, which is a id of thread.\n    DEADBEATS.reset_thread()\n```\n\n## With PyTorch Lightning\n\n```python\nfrom deadbeats import DEADBEATS\n\nclass MyModel(pl.LightningModule):\n\n    ...\n\n    def on_train_start(self):\n        DEADBEATS.start_thread()\n\n    ...\n\n    def validation_epoch_end(self, outputs):\n        avg_loss = torch.stack([x[\'val_loss\'] for x in outputs]).mean()\n\n        DEADBEATS.ping(val_loss = avg_loss, current_epoch = self.current_epoch)\n\n        return {\'val_loss\': avg_loss}\n\n    ...\n\n    # custom training function\n    @DEADBEATS.wrap\n    def fit(self, trainer):\n        trainer.fit(self)\n\n```\n\n\n## messages like below\n\n![example](.github/images/example_message.png)\n\n\n\nThis library is named after the wonderful work of [Mori Calliope](https://www.youtube.com/channel/UCL_qhgtOy0dy1Agp8vkySQg), [DEAD BEATS](https://youtu.be/6ydgEipkUEU), and inspired by [hugginface/knockknock](https://github.com/huggingface/knockknock).\n\n',
    'author': 'hppRC',
    'author_email': 'hpp.ricecake@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hpprc/deadbeats',
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
